@@ -1,19 +1,8 @@
 /*
- * Copyright 2017 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- * Copyright (C) 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2017 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+   SPDX-FileCopyrightText: 2012 (c) Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "elisaapplication.h"
@@ -45,7 +34,8 @@
 
 #include <KCoreAddons/KAboutData>
 
-#include <QQmlApplicationEngine>
+#include <QQmlEngine>
+#include <QQmlComponent>
 #include <QGuiApplication>
 #include <QDesktopServices>
 #include <QDialog>
@@ -55,6 +45,7 @@
 #include <QUrl>
 #include <QFileInfo>
 #include <QDir>
+#include <QFileSystemWatcher>
 #include <QKeyEvent>
 #include <QDebug>
 #include <QFileSystemWatcher>
@@ -71,21 +62,13 @@ public:
     #endif
     {
         Q_UNUSED(parent)
-
-        auto configurationFileName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
-        configurationFileName += QStringLiteral("/elisarc");
-        Elisa::ElisaConfiguration::instance(configurationFileName);
-        Elisa::ElisaConfiguration::self()->load();
-        Elisa::ElisaConfiguration::self()->save();
-
-        mConfigFileWatcher.addPath(configurationFileName);
     }
 
 #if defined KF5XmlGui_FOUND && KF5XmlGui_FOUND
     KActionCollection mCollection;
 #endif
 
-    ElisaUtils::EntryDataList mArguments;
+    DataTypes::EntryDataList mArguments;
 
     std::unique_ptr<MusicListenersManager> mMusicManager;
 
@@ -101,7 +84,7 @@ public:
 
     std::unique_ptr<ManageHeaderBar> mManageHeaderBar;
 
-    QQmlApplicationEngine *mEngine = nullptr;
+    QQmlEngine *mEngine = nullptr;
 
     QFileSystemWatcher mConfigFileWatcher;
 
@@ -109,6 +92,14 @@ public:
 
 ElisaApplication::ElisaApplication(QObject *parent) : QObject(parent), d(std::make_unique<ElisaApplicationPrivate>(this))
 {
+    auto configurationFileName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
+    configurationFileName += QStringLiteral("/elisarc");
+    Elisa::ElisaConfiguration::instance(configurationFileName);
+    Elisa::ElisaConfiguration::self()->load();
+    Elisa::ElisaConfiguration::self()->save();
+
+    d->mConfigFileWatcher.addPath(Elisa::ElisaConfiguration::self()->config()->name());
+
     connect(Elisa::ElisaConfiguration::self(), &Elisa::ElisaConfiguration::configChanged,
             this, &ElisaApplication::configChanged);
 
@@ -190,7 +181,7 @@ void ElisaApplication::setupActions(const QString &actionName)
 #endif
 }
 
-void ElisaApplication::setArguments(const ElisaUtils::EntryDataList &newArguments)
+void ElisaApplication::setArguments(const DataTypes::EntryDataList &newArguments)
 {
     if (d->mArguments == newArguments) {
         return;
@@ -200,7 +191,7 @@ void ElisaApplication::setArguments(const ElisaUtils::EntryDataList &newArgument
     Q_EMIT argumentsChanged();
 
     if (!d->mArguments.isEmpty()) {
-        Q_EMIT enqueue(d->mArguments, ElisaUtils::FileName,
+        Q_EMIT enqueue(d->mArguments,
                        ElisaUtils::PlayListEnqueueMode::AppendPlayList,
                        ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
@@ -215,7 +206,7 @@ void ElisaApplication::activateActionRequested(const QString &actionName, const
 void ElisaApplication::activateRequested(const QStringList &arguments, const QString &workingDirectory)
 {
     if (arguments.size() > 1) {
-        auto realArguments = ElisaUtils::EntryDataList{};
+        auto realArguments = DataTypes::EntryDataList{};
 
         bool isFirst = true;
         for (const auto &oneArgument : arguments) {
@@ -224,11 +215,10 @@ void ElisaApplication::activateRequested(const QStringList &arguments, const QSt
                 continue;
             }
 
-            realArguments.push_back(ElisaUtils::EntryData{{}, {}, QUrl(oneArgument)});
+            realArguments.push_back(DataTypes::EntryData{{}, {}, QUrl(oneArgument)});
         }
 
         Q_EMIT enqueue(checkFileListAndMakeAbsolute(realArguments, workingDirectory),
-                       ElisaUtils::FileName,
                        ElisaUtils::PlayListEnqueueMode::AppendPlayList,
                        ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
@@ -285,7 +275,10 @@ void ElisaApplication::configureElisa()
         return;
     }
 
-    d->mEngine->load(QUrl(QStringLiteral("qrc:/qml/ElisaConfigurationDialog.qml")));
+    QQmlComponent configurationDialogComponent(d->mEngine,
+                                               QUrl(QStringLiteral("qrc:/qml/ElisaConfigurationDialog.qml")));
+
+    configurationDialogComponent.create();
 }
 
 void ElisaApplication::goBack() {}
@@ -311,12 +304,13 @@ void ElisaApplication::configChanged()
 
     Q_EMIT showProgressOnTaskBarChanged();
     Q_EMIT showSystemTrayIconChanged();
+    Q_EMIT embeddedViewChanged();
 }
 
-ElisaUtils::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const ElisaUtils::EntryDataList &filesList,
+DataTypes::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const DataTypes::EntryDataList &filesList,
                                                                          const QString &workingDirectory) const
 {
-    auto filesToOpen = ElisaUtils::EntryDataList{};
+    auto filesToOpen = DataTypes::EntryDataList{};
 
     for (const auto &oneFile : filesList) {
         if (std::get<2>(oneFile).scheme().isEmpty() || std::get<2>(oneFile).isLocalFile()) {
@@ -334,10 +328,10 @@ ElisaUtils::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const E
             }
 
             if (newFile.exists()) {
-                filesToOpen.push_back(ElisaUtils::EntryData{{}, {}, QUrl::fromLocalFile(newFile.canonicalFilePath())});
+                filesToOpen.push_back(DataTypes::EntryData{{}, {}, QUrl::fromLocalFile(newFile.canonicalFilePath())});
             }
         } else {
-            filesToOpen.push_back(ElisaUtils::EntryData{{}, {}, std::get<2>(oneFile)});
+            filesToOpen.push_back(DataTypes::EntryData{{}, {}, std::get<2>(oneFile)});
         }
     }
 
@@ -352,7 +346,7 @@ void ElisaApplication::initialize()
     Q_EMIT initializationDone();
 }
 
-void ElisaApplication::setQmlEngine(QQmlApplicationEngine *engine)
+void ElisaApplication::setQmlEngine(QQmlEngine *engine)
 {
     d->mEngine = engine;
 }
@@ -373,10 +367,9 @@ void ElisaApplication::initializeModels()
     d->mMusicManager->setElisaApplication(this);
 
     QObject::connect(this, &ElisaApplication::enqueue,
-                     d->mMediaPlayListProxyModel.get(), static_cast<void (MediaPlayListProxyModel::*)(const ElisaUtils::EntryDataList&,
-                                                                                  ElisaUtils::PlayListEntryType,
-                                                                                  ElisaUtils::PlayListEnqueueMode,
-                                                                                  ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayListProxyModel::enqueue));
+                     d->mMediaPlayListProxyModel.get(), static_cast<void (MediaPlayListProxyModel::*)(const DataTypes::EntryDataList&,
+                                                                                            ElisaUtils::PlayListEnqueueMode,
+                                                                                            ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayListProxyModel::enqueue));
 }
 
 void ElisaApplication::initializePlayer()
@@ -451,7 +444,7 @@ void ElisaApplication::initializePlayer()
     QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::currentTrackDataChanged, d->mManageHeaderBar.get(), &ManageHeaderBar::updateCurrentTrackData);
 
     if (!d->mArguments.isEmpty()) {
-        Q_EMIT enqueue(d->mArguments, ElisaUtils::FileName,
+        Q_EMIT enqueue(d->mArguments,
                        ElisaUtils::PlayListEnqueueMode::AppendPlayList,
                        ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
@@ -491,7 +484,7 @@ void ElisaApplication::installKeyEventFilter(QObject *object)
 
 bool ElisaApplication::eventFilter(QObject *object, QEvent *event)
 {
-    Q_UNUSED(object);
+    Q_UNUSED(object)
 
     QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
     auto playPauseAction = d->mCollection.action(tr("Play-Pause"));
@@ -502,7 +495,7 @@ bool ElisaApplication::eventFilter(QObject *object, QEvent *event)
     return false;
 }
 
-const ElisaUtils::EntryDataList &ElisaApplication::arguments() const
+const DataTypes::EntryDataList &ElisaApplication::arguments() const
 {
     return d->mArguments;
 }
@@ -557,4 +550,27 @@ bool ElisaApplication::showSystemTrayIcon() const
     return currentConfiguration->showSystemTrayIcon();
 }
 
+ElisaUtils::PlayListEntryType ElisaApplication::embeddedView() const
+{
+    ElisaUtils::PlayListEntryType result = ElisaUtils::Unknown;
+
+    switch (Elisa::ElisaConfiguration::self()->embeddedView())
+    {
+    case 0:
+        result = ElisaUtils::Unknown;
+        break;
+    case 1:
+        result = ElisaUtils::Album;
+        break;
+    case 2:
+        result = ElisaUtils::Artist;
+        break;
+    case 3:
+        result = ElisaUtils::Genre;
+        break;
+    }
+
+    return result;
+}
+
 #include "moc_elisaapplication.cpp"
