@@ -1,23 +1,13 @@
 /*
- * Copyright 2015-2018 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2015 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "config-upnp-qt.h"
 #include "elisa-version.h"
 
+#include "elisaarguments.h"
 #include "elisaapplication.h"
 #include "elisa_settings.h"
 
@@ -39,10 +29,6 @@
 #include <KCrash>
 #endif
 
-#if defined KF5DBusAddons_FOUND && KF5DBusAddons_FOUND
-#include <KDBusService>
-#endif
-
 
 #include <QIcon>
 
@@ -51,6 +37,7 @@
 #include <QStandardPaths>
 
 #include <QQmlApplicationEngine>
+#include <QJSEngine>
 #include <QQmlFileSelector>
 #include <QQuickStyle>
 #include <QQmlContext>
@@ -153,31 +140,25 @@ int main(int argc, char *argv[])
 
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
 
-#if defined KF5DBusAddons_FOUND && KF5DBusAddons_FOUND
-    KDBusService elisaService(KDBusService::Unique);
-#endif
-
-    std::unique_ptr<ElisaApplication> myApp = std::make_unique<ElisaApplication>();
-
-    myApp->setQmlEngine(&engine);
-
-#if defined KF5DBusAddons_FOUND && KF5DBusAddons_FOUND
-    QObject::connect(&elisaService, &KDBusService::activateActionRequested, myApp.get(), &ElisaApplication::activateActionRequested);
-    QObject::connect(&elisaService, &KDBusService::activateRequested, myApp.get(), &ElisaApplication::activateRequested);
-    QObject::connect(&elisaService, &KDBusService::openRequested, myApp.get(), &ElisaApplication::openRequested);
-#endif
-
-    auto arguments = ElisaUtils::EntryDataList{};
+    auto arguments = DataTypes::EntryDataList{};
     auto realArgumentsList = parser.positionalArguments();
 
     for (const auto &oneArgument : realArgumentsList) {
-        arguments.push_back(ElisaUtils::EntryData{{}, {}, QUrl(oneArgument)});
+        arguments.push_back(DataTypes::EntryData{{}, {}, QUrl(oneArgument)});
     }
 
-    myApp->setArguments(arguments);
+    int typeId = qmlRegisterSingletonType<ElisaArguments>("org.kde.elisa.host", 1, 0, "ElisaArguments", [](QQmlEngine *qmlEngine, QJSEngine *scriptEngine) -> QObject* {
+        Q_UNUSED(qmlEngine)
+        Q_UNUSED(scriptEngine)
 
-    engine.rootContext()->setContextProperty(QStringLiteral("elisa"), myApp.release());
-    engine.rootContext()->setContextProperty(QStringLiteral("config"), new ElisaConfigurationDialog);
+        auto *result = new ElisaArguments;
+
+        return result;
+    });
+
+    auto *argumentsSingleton = engine.singletonInstance<ElisaArguments*>(typeId);
+
+    argumentsSingleton->setArguments(arguments);
 
     engine.load(QUrl(QStringLiteral("qrc:/qml/ElisaMainWindow.qml")));
 
