@@ -1,19 +1,8 @@
 /*
- * Copyright 2015-2017 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- * Copyright 2019-2020 Alexander Stippich <a.stippich@gmx.net>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program. If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2015 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+   SPDX-FileCopyrightText: 2019 (c) Alexander Stippich <a.stippich@gmx.net>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "mediaplaylistproxymodel.h"
@@ -25,6 +14,8 @@
 #include <QRandomGenerator>
 #include <QFile>
 #include <QFileInfo>
+#include <QDir>
+#include <QMimeDatabase>
 
 #include <algorithm>
 
@@ -48,6 +39,8 @@ public:
 
     QRandomGenerator mRandomGenerator;
 
+    QMimeDatabase mMimeDb;
+
     ElisaUtils::PlayListEnqueueTriggerPlay mTriggerPlay = ElisaUtils::DoNotTriggerPlay;
 
     int mCurrentPlayListPosition = -1;
@@ -316,14 +309,14 @@ void MediaPlayListProxyModel::sourceRowsInserted(const QModelIndex &parent, int
         if (rowCount() == 0 || newItemsCount == 1) {
             beginInsertRows(parent, start, end);
             for (int i = 0; i < newItemsCount; ++i) {
-                //QRandomGenerator.bounded(int) is exlusive, thus + 1
+                //QRandomGenerator.bounded(int) is exclusive, thus + 1
                 const auto random = d->mRandomGenerator.bounded(d->mRandomMapping.count()+1);
                 d->mRandomMapping.insert(random, start + i);
             }
             endInsertRows();
         } else {
             for (int i = 0; i < newItemsCount; ++i) {
-                //QRandomGenerator.bounded(int) is exlusive, thus + 1
+                //QRandomGenerator.bounded(int) is exclusive, thus + 1
                 const auto random = d->mRandomGenerator.bounded(d->mRandomMapping.count()+1);
                 beginInsertRows(parent, random, random);
                 d->mRandomMapping.insert(random, start + i);
@@ -477,74 +470,20 @@ int MediaPlayListProxyModel::currentTrackRow() const
     return d->mCurrentTrack.row();
 }
 
-void MediaPlayListProxyModel::enqueue(const ElisaUtils::EntryData &newEntry, ElisaUtils::PlayListEntryType databaseIdType)
+void MediaPlayListProxyModel::enqueue(const DataTypes::MusicDataType &newEntry, const QString &newEntryTitle,
+                                      ElisaUtils::PlayListEnqueueMode enqueueMode, ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay)
 {
-    enqueue(newEntry, databaseIdType, ElisaUtils::PlayListEnqueueMode::AppendPlayList, ElisaUtils::PlayListEnqueueTriggerPlay::DoNotTriggerPlay);
+    enqueue({{newEntry, newEntryTitle, {}}}, enqueueMode, triggerPlay);
 }
 
-void MediaPlayListProxyModel::enqueue(const ElisaUtils::EntryDataList &newEntries, ElisaUtils::PlayListEntryType databaseIdType)
-{
-    enqueue(newEntries, databaseIdType, ElisaUtils::PlayListEnqueueMode::AppendPlayList, ElisaUtils::PlayListEnqueueTriggerPlay::DoNotTriggerPlay);
-}
-
-void MediaPlayListProxyModel::enqueue(qulonglong newEntryDatabaseId,
-                            const QString &newEntryTitle,
-                            ElisaUtils::PlayListEntryType databaseIdType,
+void MediaPlayListProxyModel::enqueue(const QUrl &entryUrl,
                             ElisaUtils::PlayListEnqueueMode enqueueMode,
                             ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay)
 {
-    enqueue(ElisaUtils::EntryData{{{DataTypes::DatabaseIdRole, newEntryDatabaseId}}, newEntryTitle, {}}, databaseIdType, enqueueMode, triggerPlay);
+    enqueue({{{{DataTypes::ElementTypeRole, ElisaUtils::Track}}, {}, entryUrl}}, enqueueMode, triggerPlay);
 }
 
-void MediaPlayListProxyModel::enqueue(const QUrl &entryUrl, ElisaUtils::PlayListEntryType databaseIdType,
-                            ElisaUtils::PlayListEnqueueMode enqueueMode,
-                            ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay)
-{
-    enqueue(ElisaUtils::EntryData{{}, {}, entryUrl}, databaseIdType, enqueueMode, triggerPlay);
-}
-
-void MediaPlayListProxyModel::enqueue(const ElisaUtils::EntryData &newEntry,
-                            ElisaUtils::PlayListEntryType databaseIdType,
-                            ElisaUtils::PlayListEnqueueMode enqueueMode,
-                            ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay)
-{
-    d->mTriggerPlay = triggerPlay;
-
-    if (enqueueMode == ElisaUtils::ReplacePlayList) {
-        if (rowCount() == 0) {
-            Q_EMIT hideUndoNotification();
-        } else {
-            clearPlayList();
-        }
-    }
-
-    switch (databaseIdType)
-    {
-    case ElisaUtils::Album:
-    case ElisaUtils::Artist:
-    case ElisaUtils::Genre:
-        d->mPlayListModel->enqueueOneEntry(newEntry, databaseIdType);
-        break;
-    case ElisaUtils::Track:
-    case ElisaUtils::Radio:
-        if (std::get<2>(newEntry).isValid()) {
-            d->mPlayListModel->enqueueFilesList({newEntry}, databaseIdType);
-        } else {
-            d->mPlayListModel->enqueueOneEntry(newEntry, databaseIdType);
-        }
-        break;
-    case ElisaUtils::FileName:
-        d->mPlayListModel->enqueueFilesList({newEntry}, databaseIdType);
-        break;
-    case ElisaUtils::Lyricist:
-    case ElisaUtils::Composer:
-    case ElisaUtils::Unknown:
-        break;
-    }
-}
-
-void MediaPlayListProxyModel::enqueue(const ElisaUtils::EntryDataList &newEntries,
-                            ElisaUtils::PlayListEntryType databaseIdType,
+void MediaPlayListProxyModel::enqueue(const DataTypes::EntryDataList &newEntries,
                             ElisaUtils::PlayListEnqueueMode enqueueMode,
                             ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay)
 {
@@ -562,25 +501,7 @@ void MediaPlayListProxyModel::enqueue(const ElisaUtils::EntryDataList &newEntrie
         }
     }
 
-    switch (databaseIdType)
-    {
-    case ElisaUtils::Track:
-    case ElisaUtils::Radio:
-        d->mPlayListModel->enqueueMultipleEntries(newEntries, databaseIdType);
-        break;
-    case ElisaUtils::FileName:
-        d->mPlayListModel->enqueueFilesList(newEntries, databaseIdType);
-        break;
-    case ElisaUtils::Album:
-    case ElisaUtils::Artist:
-    case ElisaUtils::Genre:
-        d->mPlayListModel->enqueueMultipleEntries(newEntries, databaseIdType);
-        break;
-    case ElisaUtils::Lyricist:
-    case ElisaUtils::Composer:
-    case ElisaUtils::Unknown:
-        break;
-    }
+    d->mPlayListModel->enqueueMultipleEntries(newEntries);
 }
 
 void MediaPlayListProxyModel::trackInError(const QUrl &sourceInError, QMediaPlayer::Error playerError)
@@ -782,14 +703,18 @@ void MediaPlayListProxyModel::loadPlayList(const QUrl &fileName)
 void MediaPlayListProxyModel::loadPlayListLoaded()
 {
     clearPlayList();
+
+    auto newTracks = DataTypes::EntryDataList{};
     for (int i = 0; i < d->mLoadPlaylist.mediaCount(); ++i) {
 #if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-        enqueue(ElisaUtils::EntryData{{}, {}, d->mLoadPlaylist.media(i).canonicalUrl()}, ElisaUtils::FileName);
+        newTracks.push_back({{{{DataTypes::ElementTypeRole, ElisaUtils::FileName}}}, {}, d->mLoadPlaylist.media(i).canonicalUrl()});
 #else
-        enqueue(ElisaUtils::EntryData{{}, {}, d->mLoadPlaylist.media(i).request().url()}, ElisaUtils::FileName);
+        newTracks.push_back({{{{DataTypes::ElementTypeRole, ElisaUtils::FileName}}}, {}, d->mLoadPlaylist.media(i).request().url()});
 #endif
     }
 
+    enqueue(newTracks, ElisaUtils::ReplacePlayList, ElisaUtils::DoNotTriggerPlay);
+
     Q_EMIT persistentStateChanged();
 
     d->mLoadPlaylist.clear();
@@ -844,4 +769,38 @@ void MediaPlayListProxyModel::setPersistentState(const QVariantMap &persistentSt
     Q_EMIT persistentStateChanged();
 }
 
+void MediaPlayListProxyModel::enqueueDirectory(const QUrl &fileName, ElisaUtils::PlayListEntryType databaseIdType,
+                                            ElisaUtils::PlayListEnqueueMode enqueueMode,
+                                            ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay, int depth)
+{
+    if (!fileName.isLocalFile()) return;
+    // clear playlist if required
+    if (enqueueMode == ElisaUtils::ReplacePlayList) {
+        if (rowCount() == 0) {
+            Q_EMIT hideUndoNotification();
+        } else {
+            clearPlayList();
+        }
+    }
+    // get contents of directory
+    QDir dirInfo = QDir(fileName.toLocalFile());
+    auto files = dirInfo.entryInfoList(QDir::NoDotAndDotDot | QDir::Readable | QDir::Files | QDir::Dirs, QDir::Name);
+    auto newFiles = DataTypes::EntryDataList();
+    for (const auto &file : files)
+    {
+        auto fileUrl = QUrl::fromLocalFile(file.filePath());
+        if (file.isFile() && d->mMimeDb.mimeTypeForUrl(fileUrl).name().startsWith(QLatin1String("audio/")))
+        {
+            newFiles.push_back({{{DataTypes::ElementTypeRole, ElisaUtils::Track},
+                                 {DataTypes::ResourceRole, fileUrl}},{}, {}});
+        }
+        else if (file.isDir() && depth > 1)
+        {
+            // recurse through directory
+            enqueueDirectory(fileUrl, databaseIdType, ElisaUtils::AppendPlayList, triggerPlay, depth-1);
+        }
+    }
+    if (newFiles.size() != 0) enqueue(newFiles, ElisaUtils::AppendPlayList, triggerPlay);
+}
+
 #include "moc_mediaplaylistproxymodel.cpp"
