@@ -1,18 +1,7 @@
 /*
- * Copyright 2015-2017 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2015 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #ifndef DATAMODEL_H
@@ -140,7 +129,12 @@ public Q_SLOTS:
 
     void initialize(MusicListenersManager *manager, DatabaseInterface *database,
                     ElisaUtils::PlayListEntryType modelType, ElisaUtils::FilterType filter,
-                    const QString &genre, const QString &artist, qulonglong databaseId);
+                    const QString &genre, const QString &artist, qulonglong databaseId,
+                    const QUrl &pathFilter);
+
+    void initializeByData(MusicListenersManager *manager, DatabaseInterface *database,
+                          ElisaUtils::PlayListEntryType modelType, ElisaUtils::FilterType filter,
+                          const DataTypes::DataType &dataFilter);
 
 private Q_SLOTS:
 
